SUPPORT_MODULE_RSHAPES: int = 1
SUPPORT_MODULE_RTEXTURES: int = 1
SUPPORT_MODULE_RTEXT: int = 1  # WARNING: It requires SUPPORT_MODULE_RTEXTURES to load sprite font textures
SUPPORT_MODULE_RMODELS: int = 1
SUPPORT_MODULE_RAUDIO: int = 1
SUPPORT_CAMERA_SYSTEM: int = 1
SUPPORT_GESTURES_SYSTEM: int = 1
SUPPORT_MOUSE_GESTURES: int = 1
SUPPORT_SSH_KEYBOARD_RPI: int = 1
SUPPORT_WINMM_HIGHRES_TIMER: int = 1
SUPPORT_SCREEN_CAPTURE: int = 1
SUPPORT_GIF_RECORDING: int = 1
SUPPORT_COMPRESSION_API: int = 1
MAX_FILEPATH_CAPACITY: int = 8192  # Maximum file paths capacity
MAX_FILEPATH_LENGTH: int = 4096  # Maximum length for filepaths (Linux PATH_MAX default value)
MAX_KEYBOARD_KEYS: int = 512  # Maximum number of keyboard keys supported
MAX_MOUSE_BUTTONS: int = 8  # Maximum number of mouse buttons supported
MAX_GAMEPADS: int = 4  # Maximum number of gamepads supported
MAX_GAMEPAD_AXIS: int = 8  # Maximum number of axis supported (per gamepad)
MAX_GAMEPAD_BUTTONS: int = 32  # Maximum number of buttons supported (per gamepad)
MAX_TOUCH_POINTS: int = 8  # Maximum number of touch points supported
MAX_KEY_PRESSED_QUEUE: int = 16  # Maximum number of keys in the key input queue
MAX_CHAR_PRESSED_QUEUE: int = 16  # Maximum number of characters in the char input queue
MAX_DECOMPRESSION_SIZE: int = 64  # Max size allocated for decompression in MB
RL_DEFAULT_BATCH_BUFFERS: int = 1  # Default number of batch buffers (multi-buffering)
RL_DEFAULT_BATCH_DRAWCALLS: int = 256  # Default number of batch draw calls (by state changes: mode, texture)
RL_DEFAULT_BATCH_MAX_TEXTURE_UNITS: int = 4  # Maximum number of textures units that can be activated on batch drawing (SetShaderValueTexture())
RL_MAX_MATRIX_STACK_SIZE: int = 32  # Maximum size of internal Matrix stack
RL_MAX_SHADER_LOCATIONS: int = 32  # Maximum number of shader locations supported
RL_CULL_DISTANCE_NEAR: float = 0.01  # Default projection matrix near cull distance
RL_CULL_DISTANCE_FAR: float = 1000.0  # Default projection matrix far cull distance
RL_DEFAULT_SHADER_ATTRIB_NAME_POSITION: str = "vertexPosition"  # Binded by default to shader location: 0
RL_DEFAULT_SHADER_ATTRIB_NAME_TEXCOORD: str = "vertexTexCoord"  # Binded by default to shader location: 1
RL_DEFAULT_SHADER_ATTRIB_NAME_NORMAL: str = "vertexNormal"  # Binded by default to shader location: 2
RL_DEFAULT_SHADER_ATTRIB_NAME_COLOR: str = "vertexColor"  # Binded by default to shader location: 3
RL_DEFAULT_SHADER_ATTRIB_NAME_TANGENT: str = "vertexTangent"  # Binded by default to shader location: 4
RL_DEFAULT_SHADER_ATTRIB_NAME_TEXCOORD2: str = "vertexTexCoord2"  # Binded by default to shader location: 5
RL_DEFAULT_SHADER_UNIFORM_NAME_MVP: str = "mvp"  # model-view-projection matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_VIEW: str = "matView"  # view matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_PROJECTION: str = "matProjection"  # projection matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_MODEL: str = "matModel"  # model matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_NORMAL: str = "matNormal"  # normal matrix (transpose(inverse(matModelView))
RL_DEFAULT_SHADER_UNIFORM_NAME_COLOR: str = "colDiffuse"  # color diffuse (base tint color, multiplied by texture color)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE0: str = "texture0"  # texture0 (texture slot active 0)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE1: str = "texture1"  # texture1 (texture slot active 1)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE2: str = "texture2"  # texture2 (texture slot active 2)
SUPPORT_QUADS_DRAW_MODE: int = 1
SUPPORT_FILEFORMAT_PNG: int = 1
SUPPORT_FILEFORMAT_GIF: int = 1
SUPPORT_FILEFORMAT_QOI: int = 1
SUPPORT_FILEFORMAT_DDS: int = 1
SUPPORT_FILEFORMAT_HDR: int = 1
SUPPORT_IMAGE_EXPORT: int = 1
SUPPORT_IMAGE_GENERATION: int = 1
SUPPORT_IMAGE_MANIPULATION: int = 1
SUPPORT_DEFAULT_FONT: int = 1
SUPPORT_FILEFORMAT_FNT: int = 1
SUPPORT_FILEFORMAT_TTF: int = 1
SUPPORT_TEXT_MANIPULATION: int = 1
MAX_TEXT_BUFFER_LENGTH: int = 1024  # Size of internal static buffers used on some functions:
MAX_TEXTSPLIT_COUNT: int = 128  # Maximum number of substrings to split: TextSplit()
SUPPORT_FILEFORMAT_OBJ: int = 1
SUPPORT_FILEFORMAT_MTL: int = 1
SUPPORT_FILEFORMAT_IQM: int = 1
SUPPORT_FILEFORMAT_GLTF: int = 1
SUPPORT_FILEFORMAT_VOX: int = 1
SUPPORT_MESH_GENERATION: int = 1
MAX_MATERIAL_MAPS: int = 12  # Maximum number of shader maps supported
MAX_MESH_VERTEX_BUFFERS: int = 7  # Maximum vertex buffers (VBO) per mesh
SUPPORT_FILEFORMAT_WAV: int = 1
SUPPORT_FILEFORMAT_OGG: int = 1
SUPPORT_FILEFORMAT_XM: int = 1
SUPPORT_FILEFORMAT_MOD: int = 1
SUPPORT_FILEFORMAT_MP3: int = 1
AUDIO_DEVICE_CHANNELS: int = 2  # Device output channels: stereo
AUDIO_DEVICE_SAMPLE_RATE: int = 0  # Device sample rate (device default)
MAX_AUDIO_BUFFER_POOL_CHANNELS: int = 16  # Maximum number of audio pool channels
SUPPORT_TRACELOG: int = 1
MAX_TRACELOG_MSG_LENGTH: int = 128  # Max length of one trace-log message
RLGL_VERSION: str = "4.0"
RL_DEFAULT_BATCH_BUFFER_ELEMENTS: int = 8192
RL_TEXTURE_WRAP_S: int = 10242  # GL_TEXTURE_WRAP_S
RL_TEXTURE_WRAP_T: int = 10243  # GL_TEXTURE_WRAP_T
RL_TEXTURE_MAG_FILTER: int = 10240  # GL_TEXTURE_MAG_FILTER
RL_TEXTURE_MIN_FILTER: int = 10241  # GL_TEXTURE_MIN_FILTER
RL_TEXTURE_FILTER_NEAREST: int = 9728  # GL_NEAREST
RL_TEXTURE_FILTER_LINEAR: int = 9729  # GL_LINEAR
RL_TEXTURE_FILTER_MIP_NEAREST: int = 9984  # GL_NEAREST_MIPMAP_NEAREST
RL_TEXTURE_FILTER_NEAREST_MIP_LINEAR: int = 9986  # GL_NEAREST_MIPMAP_LINEAR
RL_TEXTURE_FILTER_LINEAR_MIP_NEAREST: int = 9985  # GL_LINEAR_MIPMAP_NEAREST
RL_TEXTURE_FILTER_MIP_LINEAR: int = 9987  # GL_LINEAR_MIPMAP_LINEAR
RL_TEXTURE_FILTER_ANISOTROPIC: int = 12288  # Anisotropic filter (custom identifier)
RL_TEXTURE_WRAP_REPEAT: int = 10497  # GL_REPEAT
RL_TEXTURE_WRAP_CLAMP: int = 33071  # GL_CLAMP_TO_EDGE
RL_TEXTURE_WRAP_MIRROR_REPEAT: int = 33648  # GL_MIRRORED_REPEAT
RL_TEXTURE_WRAP_MIRROR_CLAMP: int = 34626  # GL_MIRROR_CLAMP_EXT
RL_MODELVIEW: int = 5888  # GL_MODELVIEW
RL_PROJECTION: int = 5889  # GL_PROJECTION
RL_TEXTURE: int = 5890  # GL_TEXTURE
RL_LINES: int = 1  # GL_LINES
RL_TRIANGLES: int = 4  # GL_TRIANGLES
RL_QUADS: int = 7  # GL_QUADS
RL_UNSIGNED_BYTE: int = 5121  # GL_UNSIGNED_BYTE
RL_FLOAT: int = 5126  # GL_FLOAT
RL_STREAM_DRAW: int = 35040  # GL_STREAM_DRAW
RL_STREAM_READ: int = 35041  # GL_STREAM_READ
RL_STREAM_COPY: int = 35042  # GL_STREAM_COPY
RL_STATIC_DRAW: int = 35044  # GL_STATIC_DRAW
RL_STATIC_READ: int = 35045  # GL_STATIC_READ
RL_STATIC_COPY: int = 35046  # GL_STATIC_COPY
RL_DYNAMIC_DRAW: int = 35048  # GL_DYNAMIC_DRAW
RL_DYNAMIC_READ: int = 35049  # GL_DYNAMIC_READ
RL_DYNAMIC_COPY: int = 35050  # GL_DYNAMIC_COPY
RL_FRAGMENT_SHADER: int = 35632  # GL_FRAGMENT_SHADER
RL_VERTEX_SHADER: int = 35633  # GL_VERTEX_SHADER
RL_COMPUTE_SHADER: int = 37305  # GL_COMPUTE_SHADER
PI: float = 3.141592653589793
DEG2RAD: float = (PI/180.0)
RAD2DEG: float = (180.0/PI)
GL_SHADING_LANGUAGE_VERSION: int = 35724
GL_COMPRESSED_RGB_S3TC_DXT1_EXT: int = 33776
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT: int = 33777
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT: int = 33778
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT: int = 33779
GL_ETC1_RGB8_OES: int = 36196
GL_COMPRESSED_RGB8_ETC2: int = 37492
GL_COMPRESSED_RGBA8_ETC2_EAC: int = 37496
GL_COMPRESSED_RGB_PVRTC_4BPPV1_IMG: int = 35840
GL_COMPRESSED_RGBA_PVRTC_4BPPV1_IMG: int = 35842
GL_COMPRESSED_RGBA_ASTC_4x4_KHR: int = 37808
GL_COMPRESSED_RGBA_ASTC_8x8_KHR: int = 37815
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT: int = 34047
GL_TEXTURE_MAX_ANISOTROPY_EXT: int = 34046
GL_UNSIGNED_SHORT_5_6_5: int = 33635
GL_UNSIGNED_SHORT_5_5_5_1: int = 32820
GL_UNSIGNED_SHORT_4_4_4_4: int = 32819
GL_LUMINANCE: int = 6409
GL_LUMINANCE_ALPHA: int = 6410
RAYLIB_VERSION: str = "4.2"
EPSILON: float = 1e-06
RAYGUI_VERSION: str = "3.2"
SCROLLBAR_LEFT_SIDE: int = 0
SCROLLBAR_RIGHT_SIDE: int = 1
RAYGUI_ICON_SIZE: int = 16  # Size of icons in pixels (squared)
RAYGUI_ICON_MAX_ICONS: int = 256  # Maximum number of icons
RAYGUI_ICON_MAX_NAME_LENGTH: int = 32  # Maximum length of icon name id
RAYGUI_MAX_CONTROLS: int = 16  # Maximum number of standard controls
RAYGUI_MAX_PROPS_BASE: int = 16  # Maximum number of standard properties
RAYGUI_MAX_PROPS_EXTENDED: int = 8  # Maximum number of extended properties
KEY_RIGHT: int = 262
KEY_LEFT: int = 263
KEY_DOWN: int = 264
KEY_UP: int = 265
KEY_BACKSPACE: int = 259
KEY_ENTER: int = 257
RAYGUI_WINDOWBOX_STATUSBAR_HEIGHT: int = 24
RAYGUI_GROUPBOX_LINE_THICK: int = 1
RAYGUI_LINE_MARGIN_TEXT: int = 12
RAYGUI_LINE_TEXT_PADDING: int = 4
RAYGUI_PANEL_BORDER_WIDTH: int = 1
RAYGUI_TOGGLEGROUP_MAX_ITEMS: int = 32
RAYGUI_VALUEBOX_MAX_CHARS: int = 32
RAYGUI_COLORBARALPHA_CHECKED_SIZE: int = 10
RAYGUI_MESSAGEBOX_BUTTON_HEIGHT: int = 24
RAYGUI_MESSAGEBOX_BUTTON_PADDING: int = 12
RAYGUI_TEXTINPUTBOX_BUTTON_HEIGHT: int = 28
RAYGUI_TEXTINPUTBOX_BUTTON_PADDING: int = 12
RAYGUI_TEXTINPUTBOX_HEIGHT: int = 28
RAYGUI_GRID_ALPHA: float = 0.15
MAX_LINE_BUFFER_SIZE: int = 256
ICON_TEXT_PADDING: int = 4
RAYGUI_TEXTSPLIT_MAX_ITEMS: int = 128
RAYGUI_TEXTSPLIT_MAX_TEXT_SIZE: int = 1024
RAYGUI_TEXTFORMAT_MAX_SIZE: int = 256



import ctypes
import json
import pathlib

import inflection

import raypyc.structures
from raypyc.colors import *
from raypyc.defines import *
from raypyc.enums import *
from raypyc.structures import *

# -----------------------------------------
wrapped_functions_names = []

current_module = __import__(__name__)

_rl = ctypes.cdll.LoadLibrary(str(pathlib.Path(__file__).parent / 'raylib.dll'))


# -----------------------------------------

def evaluate_c_type_string_to_ctype_type(c_type_string):
    cStringToCtypesString = {
        'bool': ctypes.c_bool,  # C type: _Bool  Python type: bool (1)
        'char': ctypes.c_char,  # C type: char  Python type: 1-character bytes object
        'wchar_t': ctypes.c_wchar,  # C type: wchar_t  Python type: 1-character string
        # 'char': c_byte,  # C type: char  Python type: int
        'unsignedchar': ctypes.c_ubyte,  # C type: unsigned char  Python type: int
        'short': ctypes.c_short,  # C type: short  Python  # type: int
        'unsignedshort': ctypes.c_ushort,  # C type: unsigned short  Python type: int
        'int': ctypes.c_int,  # C type: int  Python type: int
        'unsignedint': ctypes.c_uint,  # C type: unsigned int  Python type: int
        'long': ctypes.c_long,  # C type: long  Python type: int
        'unsignedlong': ctypes.c_ulong,  # C type: unsigned long  Python type: int
        'uint64': ctypes.c_longlong,  # C type: __int64 or long-long  Python type: int
        'unsigneduint64': ctypes.c_ulonglong,  # C type: unsigned __int64 or unsigned long-long  Python type: int
        'unsignedlonglong': ctypes.c_ulonglong,  # C type: unsigned __int64 or unsigned long-long  Python type: int
        'size_t': ctypes.c_size_t,  # C type: unsigned size_t  Python type: int
        'ssize_t': ctypes.c_ssize_t,  # C type: ssize_t or Py_ssize_t  Python type: int
        'float': ctypes.c_float,  # C type: float  Python type: float
        'double': ctypes.c_double,  # C type: double  Python type: float
        'longdouble': ctypes.c_longdouble,  # C type: long double  Python type: float
        'char*': ctypes.c_char_p,  # C type: char* (NUL terminated)  Python type: bytes object or None
        'wchar_t*': ctypes.c_wchar_p,  # C type: wchar_t* (NUL terminated)  Python type: string object or None
        'void*': ctypes.c_void_p,  # C type: wchar_t* (NUL terminated)  Python type: int or None
        'void': None,  # C type: void  Python type: None
    }

    c_type_string = c_type_string.replace(' ', '').replace('const', '')  # remove spaces, and "const" because there isn't really a const type in python...
    is_array = ']' in c_type_string
    pointer_level = c_type_string.count("*")

    if c_type_string == "...":
        return None

    if is_array and pointer_level > 0:  # array of pointers to structures
        type_of_array_string = c_type_string.split('[')[0]
        array_size = int(c_type_string.split('[')[1][:-1])
        type_of_array_without_pointers_string = type_of_array_string.replace('*', '')

        if type_of_array_string in cStringToCtypesString:  # basic type pointer (int*, char*, float*, ...)
            return cStringToCtypesString[type_of_array_string] * array_size

        if type_of_array_without_pointers_string in cStringToCtypesString:  # basic type pointer(probably double+ pointer level) (int**, char**, float**, ...)
            type_of_array_end = cStringToCtypesString[type_of_array_without_pointers_string]
            for i in range(pointer_level):
                type_of_array_end = ctypes.POINTER(type_of_array_end)
            return type_of_array_end * array_size

        if type_of_array_without_pointers_string in raypyc.structures.__structs:  # a struct array pointer level 1+ or just a pointer level 1
            type_of_array_end = raypyc.structures.__structs[type_of_array_without_pointers_string]
            for i in range(pointer_level):
                type_of_array_end = ctypes.POINTER(type_of_array_end)
            return type_of_array_end * array_size

        raise TypeError(f"can wrapped this string, the string: {c_type_string}")

    elif is_array:
        type_of_array_string = c_type_string.split('[')[0]
        array_size = int(c_type_string.split('[')[1][:-1])

        if type_of_array_string in cStringToCtypesString:  # basic type array (int, char, float, ...)
            return cStringToCtypesString[type_of_array_string] * array_size

        if type_of_array_string in raypyc.structures.__structs:  # a struct array
            return raypyc.structures.__structs[type_of_array_string] * array_size

        raise TypeError(f"can wrapped this string, the string: {c_type_string}")

    elif pointer_level > 0:
        if c_type_string in cStringToCtypesString:  # basic type pointer (int**, char*, float*, ...)
            return cStringToCtypesString[c_type_string]

        type_without_pointers_string = c_type_string.replace('*', '')

        if type_without_pointers_string in cStringToCtypesString:  # basic type pointer(probably double+ pointer level) (int**, char**, float**, ...)
            type_of_pointer_end = cStringToCtypesString[type_without_pointers_string]
            for i in range(pointer_level):
                type_of_pointer_end = ctypes.POINTER(type_of_pointer_end)
            return type_of_pointer_end

        if type_without_pointers_string in raypyc.structures.__structs:  # a struct pointer level 1+ or just a pointer level 1
            type_of_pointer_end = raypyc.structures.__structs[type_without_pointers_string]
            for i in range(pointer_level):
                type_of_pointer_end = ctypes.POINTER(type_of_pointer_end)
            return type_of_pointer_end

        raise TypeError(f"can wrapped this string, the string: {c_type_string}")

    if c_type_string in cStringToCtypesString:  # "regular" value not a pointer or an array
        return cStringToCtypesString[c_type_string]

    if c_type_string in raypyc.structures.__structs:  # a struct
        return raypyc.structures.__structs[c_type_string]

    raise TypeError(f"can wrapped this string, the string: {c_type_string}")


# get numbers from string
def get_numbers_from_string(string):
    ints = []
    for s in string.split(' '):
        if s.isdigit():
            ints.append(int(s))
    return ints


def wrap_function(funcname, _argtypes, _restype):
    func = _rl.__getattr__(funcname)
    func.argtypes = _argtypes
    func.restype = _restype
    return func


# add function to module
def add_function_to_module(_current_module, name_of_function, function):
    setattr(_current_module, name_of_function, function)


# return a set (from a functions set) of functions that can be wrap
def check_for_functions_that_can_wrap(functions_set):
    functions_that_can_be_wrap = []
    functions_that_cant_be_wrap = []
    # check if function can wrap
    for function in functions_set:
        do_wrapper_this_function = True
        if 'params' in function.keys():
            for function_param in function['params']:
                function_param_processed = function_param["type"].replace('const', '').replace(" ", "").replace("*", "")
                if function_param_processed.count("[") > 0:
                    function_param_processed = function_param_processed.split('[')[0]

                try:
                    evaluate_c_type_string_to_ctype_type(function_param["type"])
                except:
                    do_wrapper_this_function = False

                if function_param_processed in ['Sound', 'AudioCallback', 'SaveFileTextCallback', 'LoadFileTextCallback', 'TraceLogCallback', 'LoadFileDataCallback', 'SaveFileDataCallback']:
                    do_wrapper_this_function = False
                    break

        if do_wrapper_this_function:
            function_returnType_processed = function["returnType"].replace('const', '').replace(" ", "").replace("*", "")
            if function_returnType_processed.count("[") > 0:
                function_returnType_processed = function_returnType_processed.split('[')[0]

            try:
                evaluate_c_type_string_to_ctype_type(function["returnType"])
            except:
                do_wrapper_this_function = False

            if function_returnType_processed in ['AudioCallback', 'SaveFileTextCallback', 'LoadFileTextCallback', 'TraceLogCallback', 'LoadFileDataCallback', 'SaveFileDataCallback']:
                do_wrapper_this_function = False

        if do_wrapper_this_function:
            functions_that_can_be_wrap.append(function)
        else:
            functions_that_cant_be_wrap.append(function)

    return functions_that_can_be_wrap


# wrap functions to ctypes functions => to _rl ctypes functions
def wrap_functions_to_ctypes_functions_add_function_to_this_module(functions_to_wrap, _current_module):
    for function_to_wrap in functions_to_wrap:
        if function_to_wrap['name'] not in wrapped_functions_names:
            wrapped_functions_names.append(function_to_wrap['name'])
            name_of_function = inflection.underscore(function_to_wrap['name']).replace('3_d', '_3d').replace('2_d', '_2d')
            function_to_wrap_ctype = {'name': function_to_wrap['name'], 'parametersTypes': [], 'returnType': None}

            if 'params' in function_to_wrap.keys():
                for function_param in function_to_wrap['params']:
                    if function_param['type'] != "...":
                        function_to_wrap_ctype['parametersTypes'].append(evaluate_c_type_string_to_ctype_type(function_param['type']))
            function_to_wrap_ctype['returnType'] = evaluate_c_type_string_to_ctype_type(function_to_wrap['returnType'])

            f = wrap_function(function_to_wrap_ctype['name'], function_to_wrap_ctype['parametersTypes'], function_to_wrap_ctype['returnType'])
            add_function_to_module(current_module, name_of_function, f)


# load rlgl data
with open(pathlib.Path(pathlib.Path(__file__).parent / 'rlgl_api.json')) as reader:
    rlgl_api = json.load(reader)

rlgl_api_functions = rlgl_api['functions']

# load raylib data
with open(pathlib.Path(pathlib.Path(__file__).parent / 'raylib_api.json')) as reader:
    raylib_api = json.load(reader)

raylib_api_functions = raylib_api['functions']

# load raymath data
with open(pathlib.Path(pathlib.Path(__file__).parent / 'raymath_api.json')) as reader:
    raymath_api = json.load(reader)

raymath_api_functions = raymath_api['functions']

# load raygui data
with open(pathlib.Path(pathlib.Path(__file__).parent / 'raygui_api.json')) as reader:
    raygui_api = json.load(reader)

raygui_api_functions = raygui_api['functions']

rlgl_functions_to_wrapped = check_for_functions_that_can_wrap(rlgl_api_functions)
raylib_functions_to_wrapped = check_for_functions_that_can_wrap(raylib_api_functions)
raymath_functions_to_wrapped = check_for_functions_that_can_wrap(raymath_api_functions)
raygui_functions_to_wrapped = check_for_functions_that_can_wrap(raygui_api_functions)

wrap_functions_to_ctypes_functions_add_function_to_this_module(rlgl_functions_to_wrapped, current_module)
wrap_functions_to_ctypes_functions_add_function_to_this_module(raylib_functions_to_wrapped, current_module)
wrap_functions_to_ctypes_functions_add_function_to_this_module(raymath_functions_to_wrapped, current_module)
wrap_functions_to_ctypes_functions_add_function_to_this_module(raygui_functions_to_wrapped, current_module)



SUPPORT_MODULE_RSHAPES: int
SUPPORT_MODULE_RTEXTURES: int
SUPPORT_MODULE_RTEXT: int  # WARNING: It requires SUPPORT_MODULE_RTEXTURES to load sprite font textures
SUPPORT_MODULE_RMODELS: int
SUPPORT_MODULE_RAUDIO: int
SUPPORT_CAMERA_SYSTEM: int
SUPPORT_GESTURES_SYSTEM: int
SUPPORT_MOUSE_GESTURES: int
SUPPORT_SSH_KEYBOARD_RPI: int
SUPPORT_WINMM_HIGHRES_TIMER: int
SUPPORT_SCREEN_CAPTURE: int
SUPPORT_GIF_RECORDING: int
SUPPORT_COMPRESSION_API: int
MAX_FILEPATH_CAPACITY: int  # Maximum file paths capacity
MAX_FILEPATH_LENGTH: int  # Maximum length for filepaths (Linux PATH_MAX default value)
MAX_KEYBOARD_KEYS: int  # Maximum number of keyboard keys supported
MAX_MOUSE_BUTTONS: int  # Maximum number of mouse buttons supported
MAX_GAMEPADS: int  # Maximum number of gamepads supported
MAX_GAMEPAD_AXIS: int  # Maximum number of axis supported (per gamepad)
MAX_GAMEPAD_BUTTONS: int  # Maximum number of buttons supported (per gamepad)
MAX_TOUCH_POINTS: int  # Maximum number of touch points supported
MAX_KEY_PRESSED_QUEUE: int  # Maximum number of keys in the key input queue
MAX_CHAR_PRESSED_QUEUE: int  # Maximum number of characters in the char input queue
MAX_DECOMPRESSION_SIZE: int  # Max size allocated for decompression in MB
RL_DEFAULT_BATCH_BUFFERS: int  # Default number of batch buffers (multi-buffering)
RL_DEFAULT_BATCH_DRAWCALLS: int  # Default number of batch draw calls (by state changes: mode, texture)
RL_DEFAULT_BATCH_MAX_TEXTURE_UNITS: int  # Maximum number of textures units that can be activated on batch drawing (SetShaderValueTexture())
RL_MAX_MATRIX_STACK_SIZE: int  # Maximum size of internal Matrix stack
RL_MAX_SHADER_LOCATIONS: int  # Maximum number of shader locations supported
RL_CULL_DISTANCE_NEAR: float  # Default projection matrix near cull distance
RL_CULL_DISTANCE_FAR: float  # Default projection matrix far cull distance
RL_DEFAULT_SHADER_ATTRIB_NAME_POSITION: str  # Binded by default to shader location: 0
RL_DEFAULT_SHADER_ATTRIB_NAME_TEXCOORD: str  # Binded by default to shader location: 1
RL_DEFAULT_SHADER_ATTRIB_NAME_NORMAL: str  # Binded by default to shader location: 2
RL_DEFAULT_SHADER_ATTRIB_NAME_COLOR: str  # Binded by default to shader location: 3
RL_DEFAULT_SHADER_ATTRIB_NAME_TANGENT: str  # Binded by default to shader location: 4
RL_DEFAULT_SHADER_ATTRIB_NAME_TEXCOORD2: str  # Binded by default to shader location: 5
RL_DEFAULT_SHADER_UNIFORM_NAME_MVP: str  # model-view-projection matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_VIEW: str  # view matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_PROJECTION: str  # projection matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_MODEL: str  # model matrix
RL_DEFAULT_SHADER_UNIFORM_NAME_NORMAL: str  # normal matrix (transpose(inverse(matModelView))
RL_DEFAULT_SHADER_UNIFORM_NAME_COLOR: str  # color diffuse (base tint color, multiplied by texture color)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE0: str  # texture0 (texture slot active 0)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE1: str  # texture1 (texture slot active 1)
RL_DEFAULT_SHADER_SAMPLER2D_NAME_TEXTURE2: str  # texture2 (texture slot active 2)
SUPPORT_QUADS_DRAW_MODE: int
SUPPORT_FILEFORMAT_PNG: int
SUPPORT_FILEFORMAT_GIF: int
SUPPORT_FILEFORMAT_QOI: int
SUPPORT_FILEFORMAT_DDS: int
SUPPORT_FILEFORMAT_HDR: int
SUPPORT_IMAGE_EXPORT: int
SUPPORT_IMAGE_GENERATION: int
SUPPORT_IMAGE_MANIPULATION: int
SUPPORT_DEFAULT_FONT: int
SUPPORT_FILEFORMAT_FNT: int
SUPPORT_FILEFORMAT_TTF: int
SUPPORT_TEXT_MANIPULATION: int
MAX_TEXT_BUFFER_LENGTH: int  # Size of internal static buffers used on some functions:
MAX_TEXTSPLIT_COUNT: int  # Maximum number of substrings to split: TextSplit()
SUPPORT_FILEFORMAT_OBJ: int
SUPPORT_FILEFORMAT_MTL: int
SUPPORT_FILEFORMAT_IQM: int
SUPPORT_FILEFORMAT_GLTF: int
SUPPORT_FILEFORMAT_VOX: int
SUPPORT_MESH_GENERATION: int
MAX_MATERIAL_MAPS: int  # Maximum number of shader maps supported
MAX_MESH_VERTEX_BUFFERS: int  # Maximum vertex buffers (VBO) per mesh
SUPPORT_FILEFORMAT_WAV: int
SUPPORT_FILEFORMAT_OGG: int
SUPPORT_FILEFORMAT_XM: int
SUPPORT_FILEFORMAT_MOD: int
SUPPORT_FILEFORMAT_MP3: int
AUDIO_DEVICE_CHANNELS: int  # Device output channels: stereo
AUDIO_DEVICE_SAMPLE_RATE: int  # Device sample rate (device default)
MAX_AUDIO_BUFFER_POOL_CHANNELS: int  # Maximum number of audio pool channels
SUPPORT_TRACELOG: int
MAX_TRACELOG_MSG_LENGTH: int  # Max length of one trace-log message
RLGL_VERSION: str
RL_DEFAULT_BATCH_BUFFER_ELEMENTS: int
RL_TEXTURE_WRAP_S: int  # GL_TEXTURE_WRAP_S
RL_TEXTURE_WRAP_T: int  # GL_TEXTURE_WRAP_T
RL_TEXTURE_MAG_FILTER: int  # GL_TEXTURE_MAG_FILTER
RL_TEXTURE_MIN_FILTER: int  # GL_TEXTURE_MIN_FILTER
RL_TEXTURE_FILTER_NEAREST: int  # GL_NEAREST
RL_TEXTURE_FILTER_LINEAR: int  # GL_LINEAR
RL_TEXTURE_FILTER_MIP_NEAREST: int  # GL_NEAREST_MIPMAP_NEAREST
RL_TEXTURE_FILTER_NEAREST_MIP_LINEAR: int  # GL_NEAREST_MIPMAP_LINEAR
RL_TEXTURE_FILTER_LINEAR_MIP_NEAREST: int  # GL_LINEAR_MIPMAP_NEAREST
RL_TEXTURE_FILTER_MIP_LINEAR: int  # GL_LINEAR_MIPMAP_LINEAR
RL_TEXTURE_FILTER_ANISOTROPIC: int  # Anisotropic filter (custom identifier)
RL_TEXTURE_WRAP_REPEAT: int  # GL_REPEAT
RL_TEXTURE_WRAP_CLAMP: int  # GL_CLAMP_TO_EDGE
RL_TEXTURE_WRAP_MIRROR_REPEAT: int  # GL_MIRRORED_REPEAT
RL_TEXTURE_WRAP_MIRROR_CLAMP: int  # GL_MIRROR_CLAMP_EXT
RL_MODELVIEW: int  # GL_MODELVIEW
RL_PROJECTION: int  # GL_PROJECTION
RL_TEXTURE: int  # GL_TEXTURE
RL_LINES: int  # GL_LINES
RL_TRIANGLES: int  # GL_TRIANGLES
RL_QUADS: int  # GL_QUADS
RL_UNSIGNED_BYTE: int  # GL_UNSIGNED_BYTE
RL_FLOAT: int  # GL_FLOAT
RL_STREAM_DRAW: int  # GL_STREAM_DRAW
RL_STREAM_READ: int  # GL_STREAM_READ
RL_STREAM_COPY: int  # GL_STREAM_COPY
RL_STATIC_DRAW: int  # GL_STATIC_DRAW
RL_STATIC_READ: int  # GL_STATIC_READ
RL_STATIC_COPY: int  # GL_STATIC_COPY
RL_DYNAMIC_DRAW: int  # GL_DYNAMIC_DRAW
RL_DYNAMIC_READ: int  # GL_DYNAMIC_READ
RL_DYNAMIC_COPY: int  # GL_DYNAMIC_COPY
RL_FRAGMENT_SHADER: int  # GL_FRAGMENT_SHADER
RL_VERTEX_SHADER: int  # GL_VERTEX_SHADER
RL_COMPUTE_SHADER: int  # GL_COMPUTE_SHADER
PI: float
DEG2RAD: float
RAD2DEG: float
GL_SHADING_LANGUAGE_VERSION: int
GL_COMPRESSED_RGB_S3TC_DXT1_EXT: int
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT: int
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT: int
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT: int
GL_ETC1_RGB8_OES: int
GL_COMPRESSED_RGB8_ETC2: int
GL_COMPRESSED_RGBA8_ETC2_EAC: int
GL_COMPRESSED_RGB_PVRTC_4BPPV1_IMG: int
GL_COMPRESSED_RGBA_PVRTC_4BPPV1_IMG: int
GL_COMPRESSED_RGBA_ASTC_4x4_KHR: int
GL_COMPRESSED_RGBA_ASTC_8x8_KHR: int
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT: int
GL_TEXTURE_MAX_ANISOTROPY_EXT: int
GL_UNSIGNED_SHORT_5_6_5: int
GL_UNSIGNED_SHORT_5_5_5_1: int
GL_UNSIGNED_SHORT_4_4_4_4: int
GL_LUMINANCE: int
GL_LUMINANCE_ALPHA: int
RAYLIB_VERSION: str
EPSILON: float
RAYGUI_VERSION: str
SCROLLBAR_LEFT_SIDE: int
SCROLLBAR_RIGHT_SIDE: int
RAYGUI_ICON_SIZE: int  # Size of icons in pixels (squared)
RAYGUI_ICON_MAX_ICONS: int  # Maximum number of icons
RAYGUI_ICON_MAX_NAME_LENGTH: int  # Maximum length of icon name id
RAYGUI_MAX_CONTROLS: int  # Maximum number of standard controls
RAYGUI_MAX_PROPS_BASE: int  # Maximum number of standard properties
RAYGUI_MAX_PROPS_EXTENDED: int  # Maximum number of extended properties
KEY_RIGHT: int
KEY_LEFT: int
KEY_DOWN: int
KEY_UP: int
KEY_BACKSPACE: int
KEY_ENTER: int
RAYGUI_WINDOWBOX_STATUSBAR_HEIGHT: int
RAYGUI_GROUPBOX_LINE_THICK: int
RAYGUI_LINE_MARGIN_TEXT: int
RAYGUI_LINE_TEXT_PADDING: int
RAYGUI_PANEL_BORDER_WIDTH: int
RAYGUI_TOGGLEGROUP_MAX_ITEMS: int
RAYGUI_VALUEBOX_MAX_CHARS: int
RAYGUI_COLORBARALPHA_CHECKED_SIZE: int
RAYGUI_MESSAGEBOX_BUTTON_HEIGHT: int
RAYGUI_MESSAGEBOX_BUTTON_PADDING: int
RAYGUI_TEXTINPUTBOX_BUTTON_HEIGHT: int
RAYGUI_TEXTINPUTBOX_BUTTON_PADDING: int
RAYGUI_TEXTINPUTBOX_HEIGHT: int
RAYGUI_GRID_ALPHA: float
MAX_LINE_BUFFER_SIZE: int
ICON_TEXT_PADDING: int
RAYGUI_TEXTSPLIT_MAX_ITEMS: int
RAYGUI_TEXTSPLIT_MAX_TEXT_SIZE: int
RAYGUI_TEXTFORMAT_MAX_SIZE: int
